@@ -29,6 +29,8 @@
 #include "main-ptrace.h"        /* for nmap --packet-trace feature */
 #include "proto-arp.h"          /* for responding to ARP requests */
 #include "stack-ndpv6.h"        /* IPv6 Neighbor Discovery Protocol */
+#include "stack-arpv4.h"        /* Handle ARP resolution and requests */
+#include "rawsock-adapter.h"
 #include "proto-banner1.h"      /* for snatching banners from systems */
 #include "proto-tcp.h"          /* for TCP/IP connection table */
 #include "proto-preprocess.h"   /* quick parse of packets */
@@ -137,8 +139,9 @@ struct ThreadPair {
      */
     struct stack_src_t _src_;
 
-    unsigned char adapter_mac[6];
-    unsigned char router_mac[6];
+    macaddress_t source_mac;
+    macaddress_t router_mac_ipv4;
+    macaddress_t router_mac_ipv6;
 
     unsigned done_transmitting;
     unsigned done_receiving;
@@ -226,7 +229,7 @@ transmit_thread(void *v) /*aka. scanning_thread() */
     uint64_t *status_syn_count;
     uint64_t entropy = masscan->seed;
 
-    LOG(1, "THREAD: xmit: starting thread #%u\n", parms->nic_index);
+    LOG(1, "[+] starting transmit thread #%u\n", parms->nic_index);
 
     /* export a pointer to this variable outside this threads so
      * that the 'status' system can print the rate of syns we are
@@ -459,7 +462,7 @@ infinite:
     /*
      * Wait until the receive thread realizes the scan is over
      */
-    LOG(1, "THREAD: xmit done, waiting for receive thread to realize this\n");
+    LOG(1, "[+] transmit thread #%u complete\n", parms->nic_index);
 
     /*
      * We are done transmitting. However, response packets will take several
@@ -495,7 +498,7 @@ infinite:
 
     /* Thread is about to exit */
     parms->done_transmitting = 1;
-    LOG(1, "THREAD: xmit: stopping thread #%u\n", parms->nic_index);
+    LOG(1, "[+] exiting transmit thread #%u                    \n", parms->nic_index);
 }
 
 
@@ -535,7 +538,7 @@ receive_thread(void *v)
     struct ThreadPair *parms = (struct ThreadPair *)v;
     const struct Masscan *masscan = parms->masscan;
     struct Adapter *adapter = parms->adapter;
-    int data_link = rawsock_datalink(adapter);
+    int data_link = stack_if_datalink(adapter);
     struct Output *out;
     struct DedupTable *dedup;
     struct PcapFile *pcapfile = NULL;
@@ -560,7 +563,7 @@ receive_thread(void *v)
     *status_tcb_count = 0;
     parms->total_tcbs = status_tcb_count;
 
-    LOG(1, "THREAD: recv: starting thread #%u\n", parms->nic_index);
+    LOG(1, "[+] starting receive thread #%u\n", parms->nic_index);
     
     /* Lock this thread to a CPU. Transmit threads are on even CPUs,
      * receive threads on odd CPUs */
@@ -628,6 +631,7 @@ receive_thread(void *v)
          */
         tcpcon_set_banner_flags(tcpcon,
                 masscan->is_capture_cert,
+                masscan->is_capture_servername,
                 masscan->is_capture_html,
                 masscan->is_capture_heartbleed,
 				masscan->is_capture_ticketbleed);
@@ -709,7 +713,7 @@ receive_thread(void *v)
      * Receive packets. This is where we catch any responses and print
      * them to the terminal.
      */
-    LOG(1, "THREAD: recv: starting main loop\n");
+    LOG(2, "[+] THREAD: recv: starting main loop\n");
     while (!is_rx_done) {
         int status;
         unsigned length;
@@ -790,7 +794,7 @@ receive_thread(void *v)
              * a multicast address */
             if (is_ipv6_multicast(ip_me)) {
                 if (parsed.found == FOUND_NDPv6 && parsed.opcode == 135) {
-                    stack_handle_neighbor_solicitation(stack, &parsed, px, length);
+                    stack_ndpv6_incoming_request(stack, &parsed, px, length);
                 }
             }
             continue;
@@ -815,7 +819,7 @@ receive_thread(void *v)
                      * these packets. We need to respond to them, so that the router
                      * can then forward the packets to us. If we don't respond, we'll
                      * get no responses. */
-                    stack_handle_neighbor_solicitation(stack, &parsed, px, length);
+                    stack_ndpv6_incoming_request(stack, &parsed, px, length);
                     continue;
                 case 136: /* Neighbor Advertisment */
                     /* TODO: If doing an --ndpscan, the scanner subsystem needs to deal
@@ -839,9 +843,9 @@ receive_thread(void *v)
                      * for our IP address (as part of our user-mode TCP/IP).
                      * Since we completely bypass the TCP/IP stack, we  have to handle ARPs
                      * ourself, or the router will lose track of us.*/
-                     stack_handle_arp(stack,
+                     stack_arp_incoming_request(stack,
                                       ip_me.ipv4,
-                                      parms->adapter_mac,
+                                      parms->source_mac,
                                       px, length);
                     break;
                 case 2: /* response */
@@ -928,9 +932,9 @@ receive_thread(void *v)
 
             if (TCP_IS_SYNACK(px, parsed.transport_offset)) {
                 if (cookie != seqno_me - 1) {
+                    ipaddress_formatted_t fmt = ipaddress_fmt(ip_them);
                     LOG(2, "%s - bad cookie: ackno=0x%08x expected=0x%08x\n",
-                        ipaddress_fmt(ip_them).string,
-                        seqno_me-1, cookie);
+                        fmt.string, seqno_me-1, cookie);
                     continue;
                 }
 
@@ -975,12 +979,14 @@ receive_thread(void *v)
                         0, 0, secs, usecs, seqno_them);
                 }
             } else if (TCP_IS_FIN(px, parsed.transport_offset)) {
+                ipaddress_formatted_t fmt;
                 /*
                  * NO TCB!
                  *  This happens when we've sent a FIN, deleted our connection,
                  *  but the other side didn't get the packet.
                  */
-                LOG(4, "%s: received FIN but no TCB\n", ipaddress_fmt(ip_them).string);
+                fmt = ipaddress_fmt(ip_them);
+                LOG(4, "%s: received FIN but no TCB\n", fmt.string);
                 if (TCP_IS_RST(px, parsed.transport_offset))
                     ; /* ignore if it's own TCP flag is set */
                 else {
@@ -1014,8 +1020,9 @@ receive_thread(void *v)
 
             /* verify: syn-cookies */
             if (cookie != seqno_me - 1) {
+                ipaddress_formatted_t fmt = ipaddress_fmt(ip_them);
                 LOG(5, "%s - bad cookie: ackno=0x%08x expected=0x%08x\n",
-                    ipaddress_fmt(ip_them).string, seqno_me-1, cookie);
+                    fmt.string, seqno_me-1, cookie);
                 continue;
             }
 
@@ -1059,7 +1066,7 @@ receive_thread(void *v)
     }
 
 
-    LOG(1, "THREAD: recv: stopping thread #%u\n", parms->nic_index);
+    LOG(1, "[+] exiting receive thread #%u                    \n", parms->nic_index);
     
     /*
      * cleanup
@@ -1230,8 +1237,9 @@ main_scan(struct Masscan *masscan)
         err = masscan_initialize_adapter(
                             masscan,
                             index,
-                            parms->adapter_mac,
-                            parms->router_mac
+                            &parms->source_mac,
+                            &parms->router_mac_ipv4,
+                            &parms->router_mac_ipv6
                             );
         if (err != 0)
             exit(1);
@@ -1254,11 +1262,12 @@ main_scan(struct Masscan *masscan)
         parms->tmplset->vulncheck = vulncheck;
         template_packet_init(
                     parms->tmplset,
-                    parms->adapter_mac,
-                    parms->router_mac,
+                    parms->source_mac,
+                    parms->router_mac_ipv4,
+                    parms->router_mac_ipv6,
                     masscan->payloads.udp,
                     masscan->payloads.oproto,
-                    rawsock_datalink(masscan->nic[index].adapter),
+                    stack_if_datalink(masscan->nic[index].adapter),
                     masscan->seed);
 
         /*
@@ -1271,7 +1280,7 @@ main_scan(struct Masscan *masscan)
             masscan->nic[index].src.port.range = 1;
         }
 
-        stack = stack_create(parms->adapter_mac, &masscan->nic[index].src);
+        stack = stack_create(parms->source_mac, &masscan->nic[index].src);
         parms->stack = stack;
 
         /*
@@ -1289,22 +1298,6 @@ main_scan(struct Masscan *masscan)
          */
         signal(SIGINT, control_c_handler);
 
-
-        
-        /*
-         * Start the scanning thread.
-         * THIS IS WHERE THE PROGRAM STARTS SPEWING OUT PACKETS AT A HIGH
-         * RATE OF SPEED.
-         */
-        parms->thread_handle_xmit = pixie_begin_thread(transmit_thread, 0, parms);
-
-
-        /*
-         * Start the MATCHING receive thread. Transmit and receive threads
-         * come in matching pairs.
-         */
-        parms->thread_handle_recv = pixie_begin_thread(receive_thread, 0, parms);
-
     }
 
     /*
@@ -1317,29 +1310,51 @@ main_scan(struct Masscan *masscan)
         now = time(0);
         gmtime_s(&x, &now);
         strftime(buffer, sizeof(buffer), "%Y-%m-%d %H:%M:%S GMT", &x);
-        LOG(0, "\nStarting masscan " MASSCAN_VERSION " (http://bit.ly/14GZzcT) at %s\n", buffer);
+        LOG(0, "Starting masscan " MASSCAN_VERSION " (http://bit.ly/14GZzcT) at %s\n",
+            buffer);
 
         if (count_ports == 1 && \
             masscan->targets.ports.list->begin == Templ_ICMP_echo && \
             masscan->targets.ports.list->end == Templ_ICMP_echo)
             { /* ICMP only */
-                LOG(0, " -- forced options: -sn -n --randomize-hosts -v --send-eth\n");
+                //LOG(0, " -- forced options: -sn -n --randomize-hosts -v --send-eth\n");
                 LOG(0, "Initiating ICMP Echo Scan\n");
                 LOG(0, "Scanning %u hosts\n",(unsigned)count_ips);
              }
         else /* This could actually also be a UDP only or mixed UDP/TCP/ICMP scan */
             {
-                LOG(0, " -- forced options: -sS -Pn -n --randomize-hosts -v --send-eth\n");
+                //LOG(0, " -- forced options: -sS -Pn -n --randomize-hosts -v --send-eth\n");
                 LOG(0, "Initiating SYN Stealth Scan\n");
                 LOG(0, "Scanning %u hosts [%u port%s/host]\n",
                     (unsigned)count_ips, (unsigned)count_ports, (count_ports==1)?"":"s");
             }
     }
+    
+    /*
+     * Start all the threads
+     */
+    for (index=0; index<masscan->nic_count; index++) {
+        struct ThreadPair *parms = &parms_array[index];
+        
+        /*
+         * Start the scanning thread.
+         * THIS IS WHERE THE PROGRAM STARTS SPEWING OUT PACKETS AT A HIGH
+         * RATE OF SPEED.
+         */
+        parms->thread_handle_xmit = pixie_begin_thread(transmit_thread, 0, parms);
+
+        /*
+         * Start the MATCHING receive thread. Transmit and receive threads
+         * come in matching pairs.
+         */
+        parms->thread_handle_recv = pixie_begin_thread(receive_thread, 0, parms);
+    }
 
     /*
      * Now wait for <ctrl-c> to be pressed OR for threads to exit
      */
-    LOG(1, "THREAD: status: starting thread\n");
+    pixie_usleep(1000 * 100);
+    LOG(1, "[+] waiting for threads to finish\n");
     status_start(&status);
     status.is_infinite = masscan->is_infinite;
     while (!is_tx_done && masscan->output.is_status_updates) {
@@ -1479,7 +1494,6 @@ main_scan(struct Masscan *masscan)
         break;
     }
 
-    LOG(1, "THREAD: status: stopping thread\n");
 
     /*
      * Now cleanup everything
@@ -1491,6 +1505,9 @@ main_scan(struct Masscan *masscan)
 
         printf("%u milliseconds ellapsed\n", (unsigned)((usec_now - usec_start)/1000));
     }
+    
+    LOG(1, "[+] all threads have exited                    \n");
+
     return 0;
 }
 
@@ -1528,10 +1545,11 @@ int main(int argc, char *argv[])
      * Initialize those defaults that aren't zero
      */
     memset(masscan, 0, sizeof(*masscan));
-    masscan->blackrock_rounds = 4;
+    /* 14 rounds seem to give way better statistical distribution than 4 with a 
+    very low impact on scan rate */
+    masscan->blackrock_rounds = 14;
     masscan->output.is_show_open = 1; /* default: show syn-ack, not rst */
     masscan->output.is_status_updates = 1; /* default: show status updates */
-    masscan->seed = get_entropy(); /* entropy for randomness */
     masscan->wait = 10; /* how long to wait for responses when done */
     masscan->max_rate = 100.0; /* max rate = hundred packets-per-second */
     masscan->nic_count = 1;
@@ -1573,7 +1591,9 @@ int main(int argc, char *argv[])
      * either options or a list of IPv4 address ranges.
      */
     masscan_command_line(masscan, argc, argv);
-    
+    if (masscan->seed == 0)
+        masscan->seed = get_entropy(); /* entropy for randomness */
+
     /*
      * Load database files like "nmap-payloads" and "nmap-service-probes"
      */
@@ -1729,6 +1749,16 @@ int main(int argc, char *argv[])
         exit(1);
         break;
 
+    case Operation_Echo:
+        masscan_echo(masscan, stdout, 0);
+        exit(0);
+        break;
+
+    case Operation_EchoAll:
+        masscan_echo(masscan, stdout, 0);
+        exit(0);
+        break;
+
     case Operation_Selftest:
         /*
          * Do a regression test of all the significant units
