@@ -8,6 +8,7 @@
 #include "string_s.h"
 #include "util-malloc.h"
 #include "massip-parse.h"
+#include "logger.h"
 
 #if defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__sun__)
 #include <unistd.h>
@@ -23,6 +24,10 @@
 # define ROUNDUP(a)           ROUNDUP2((a), sizeof(int))
 #elif defined(__NetBSD__)
 # define ROUNDUP(a)           ROUNDUP2((a), sizeof(uint64_t))
+#elif defined(__FreeBSD__)
+# define ROUNDUP(a)           ROUNDUP2((a), sizeof(int))
+#elif defined(__OpenBSD__)
+# define ROUNDUP(a)           ROUNDUP2((a), sizeof(int))
 #else
 # error unknown platform
 #endif
@@ -106,7 +111,7 @@ rawsock_get_default_gateway(const char *ifname, unsigned *ipv4)
 {
     int fd;
     int seq = (int)time(0);
-    size_t err;
+    ssize_t err;
     struct rt_msghdr *rtm;
     size_t sizeof_buffer;
 
@@ -128,6 +133,21 @@ rawsock_get_default_gateway(const char *ifname, unsigned *ipv4)
         return errno;
     }
 
+    /* Needs a timeout. Sometimes it'll hang indefinitely waiting for a 
+     * response that will never arrive */
+    {
+        struct timeval timeout;
+        timeout.tv_sec = 1;
+        timeout.tv_usec = 0;
+
+        err = setsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout, sizeof(timeout));
+        if (err < 0)
+            LOG(0, "[-] SO_RCVTIMEO: %d %s\n", errno, strerror(errno));
+
+        err = setsockopt(fd, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout, sizeof(timeout));
+        if (err < 0)
+            LOG(0, "[-] SO_SNDTIMEO: %d %s\n", errno, strerror(errno));
+   }
 
     /*
      * Format and send request to kernel
@@ -140,24 +160,21 @@ rawsock_get_default_gateway(const char *ifname, unsigned *ipv4)
     rtm->rtm_pid = getpid();
     rtm->rtm_seq = seq;
 
-        /*
-         * Create an empty address
-         */
-        {
-                struct sockaddr_in *sin;
-                sin = (struct sockaddr_in *)(rtm + 1);
-                sin->sin_len = sizeof(*sin);
-                sin->sin_family = AF_INET;
-                sin->sin_addr.s_addr = 0;
-        }
+    /*
+     * Create an empty address of 0.0.0.0 to query the route
+     */
+    {
+        struct sockaddr_in *sin;
+        sin = (struct sockaddr_in *)(rtm + 1);
+        sin->sin_len = sizeof(*sin);
+        sin->sin_family = AF_INET;
+        sin->sin_addr.s_addr = 0;
+    }
 
     err = write(fd, (char *)rtm, rtm->rtm_msglen);
     if (err <= 0) {
-        perror("write(RTM_GET)");
-        printf("----%u %u\n", (unsigned)err, (unsigned)sizeof_buffer);
-        close(fd);
-        free(rtm);
-        return -1;
+        LOG(0, "[-] getroute: write(): returned %d %s\n", errno, strerror(errno));
+        goto fail;
     }
 
     /*
@@ -179,9 +196,6 @@ rawsock_get_default_gateway(const char *ifname, unsigned *ipv4)
     }
     close(fd);
 
-    //hexdump(rtm+1, err-sizeof(*rtm));
-    //dump_rt_addresses(rtm);
-
     /*
      * Parse our data
      */
@@ -209,6 +223,7 @@ rawsock_get_default_gateway(const char *ifname, unsigned *ipv4)
 
     }
 
+fail:
     free(rtm);
     return -1;
 }
